/*
    Outlet Property Precompose
        あかつきみさき (みくちぃP)

    このスクリプトについて
        選択したレイヤーをプリコンポーズし,選択していたプロパティをマスタープロパティとして設定するスクリプト.
        選択したレイヤーに調整レイヤーが存在する時は,プリコンポーズ後のレイヤーのコラップスがオンになります.

    使用方法
        ファイル→スクリプト→スクリプトファイルを実行.

    動作環境
        Adobe After Effects CC 2018以上

    バージョン情報
        2018/10/09 Ver 1.0.0 Release
*/

/// <reference path="C:/Users/RUI/OneDrive/lib/aftereffects.d.ts/ae.d.ts"/>

(function () {
    if (parseFloat(app.version) < 15.0) {
        alert("This script requires After Effects CC 2018 or greater.", "Outlet Property Precompose");
        return 0;
    }

    var actComp = app.project.activeItem;
    if (actComp == null || !(actComp instanceof CompItem)) {
        alert("Select Composition", "Outlet Property Precompose");
        return 0;
    }

    var selLayers = actComp.selectedLayers;
    if (selLayers.length == 0) {
        alert("Select Layers", "Outlet Property Precompose");
        return 0;
    }

    var selectedEnableProperty = false;
    for (var j = 0; j < actComp.selectedProperties.length; j++) {
        if (actComp.selectedProperties[j].canAddToMotionGraphicsTemplate(actComp)) {
            selectedEnableProperty = true;
            break;
        }
    }

    if (!selectedEnableProperty) {
        alert("Select properties that can be added to essential graphics", "Outlet Property Precompose");
        return 0;
    }

    var masterCompName = prompt("New composition name", "Outlet Property Precompose", "Outlet Property Precompose");
    if (masterCompName == "" || masterCompName == null) {
        return 0;
    }

    var layerIndices = [];
    var skipPropertyNames = [];
    var hasAdjustLayer = false;

    app.beginUndoGroup("Outlet Property Precompose");
    for (var i = 0; i < selLayers.length; i++) {
        if (selLayers[i] instanceof AVLayer && selLayers[i].adjustmentLayer) {
            hasAdjustLayer = true;
        }

        layerIndices[layerIndices.length] = selLayers[i].index;

        var selProperties = selLayers[i].selectedProperties;

        for (var j = 0; j < selProperties.length; j++) {
            if (selProperties[j] instanceof PropertyGroup) {
                continue;
            }

            if (!selProperties[j].canAddToMotionGraphicsTemplate(actComp)) {
                skipPropertyNames[skipPropertyNames.length] = selProperties[j].name;
                continue;
            }

            selProperties[j].addToMotionGraphicsTemplate(actComp);
        }
    }

    var labelColor = selLayers[selLayers.length - 1].label;
    var masterComp = actComp.layers.precompose(layerIndices, masterCompName, true);
    masterComp.motionGraphicsTemplateName = masterCompName;

    var masterCompLayer = actComp.selectedLayers[actComp.selectedLayers.length - 1];
    masterCompLayer.label = labelColor;

    if (hasAdjustLayer) {
        if (masterCompLayer.canSetCollapseTransformation) {
            masterCompLayer.collapseTransformation = true;
        }
    }

    masterComp.openInEssentialGraphics();
    app.endUndoGroup();

    if (skipPropertyNames.length != 0) {
        alert("Skip property\n\n" + skipPropertyNames.join("\n"), "Outlet Property Precompose");
    }

    return 0;
}).call(this);
